#!/bin/bash
# Run Javascript version of ffkernel and ffpp as one single command
# =================================================================
# 
#
#            __  __                 _            _     
#   >_<     / _|/ _|_ __   ___   __| | ___   ___| |__  
#  {0v0}   | |_| |_| '_ \ / _ \ / _` |/ _ \ / __| '_ \ 
#  (   )   |  _|  _| | | | (_) | (_| |  __/_\__ \ | | |
# --m-m--  |_| |_| |_| |_|\___/ \__,_|\___(_)___/_| |_|
#                                                      
#
# [[http://www.ljll.fr/lehyaric][Antoine Le Hyaric]]
#
# Sorbonne Université, CNRS, Laboratoire Jacques-Louis Lions, F-75005, Paris, France
#
# ----------------------------
# This file is part of FreeFEM
# ----------------------------
# FreeFEM is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
# FreeFEM is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public License along with FreeFEM; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
# [[elisp:(progn (org-link-minor-mode) (setq org-link-descriptive nil) (org-toggle-link-display))][hide links]] [[elisp:(progn (org-link-minor-mode) (setq org-link-descriptive t) (org-toggle-link-display))][show links]] [[elisp:(alh-1dex-open)][open 1dex]] [[elisp:(alh-1dex-update)][update 1dex]]
# [[elisp:(alh-set-keywords)][set keywords]] ([[file:~/alh/bin/headeralh][headeralh]])
# emacs-keywords brief="Run Javascript version of ffkernel and ffpp as one single command" freefem shell start=27/05/2024 univ update=07/06/24

# File location

ffjsdir=`dirname $0`
ffjsdir=`realpath $ffjsdir`
echo ffkernel.sh: ffjs dir is $ffjsdir

ffkernel=$ffjsdir/ffkernel.js
ffpp=$ffjsdir/ffpp.js

# Find out how to run the executable thanks to [[file:Makefile::BINTAG_type]]

optim_ok=no
if test -f $ffjsdir/BINTAG_native
then
    echo ffkernel.sh: binary type is binary
    nodejs=
    optim_ok=yes
fi

if test -f $ffjsdir/BINTAG_dev -o -f $ffjsdir/BINTAG_min
then
    echo ffkernel.sh: binary type is javascript
    nodejs=nodejs
    optim_ok=yes
fi

if test $optim_ok != yes
then
    echo ffkernel.sh: ERROR: binary type not specified
    exit 1
fi

# Running script

echo ffkernel.sh: EDP file is $1
pushd `dirname $1`
echo "$nodejs $ffpp|$nodejs $ffkernel"
cat `basename $1`|$nodejs $ffpp|$nodejs $ffkernel
popd

# Local Variables:
# eval:(visual-line-mode t)
# mode:shell-script
# coding:utf-8
# eval:(flyspell-prog-mode)
# eval:(outline-minor-mode)
# eval:(org-link-minor-mode)
# End:
# LocalWords: emacs
