// Prefix to the FF emscripten bitcode
// ===================================
// 
// 
//       \|||/                                 
//       (o o)                           _     
// ,-ooO--(_)------,   _ __  _ __ ___   (_)___ 
// |    _________  |  | '_ \| '__/ _ \  | / __|
// |   /  _._   /  |  | |_) | | |  __/_ | \__ \
// |  /________/   |  | .__/|_|  \___(_)/ |___/
// '-----------ooO-'  |_|             |__/     
// 
// Written by Antoine Le Hyaric ([[http://www.ljll.fr/lehyaric]])
// 
// Sorbonne Université, CNRS, Laboratoire Jacques-Louis Lions, F-75005, Paris, France
// 
// ----------------------------
// This file is part of FreeFEM
// ----------------------------
// FreeFEM is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
// FreeFEM is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
// You should have received a copy of the GNU Lesser General Public License along with FreeFEM; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// 
// _references_ [[ffjs_ready]]
// _links_ [[ffjs_files_data]] [[ffjs_output]]
// _TODO_ [[file:::49]]
// [[elisp:(progn (org-link-minor-mode) (setq org-link-descriptive nil) (org-toggle-link-display))][hide links]] [[elisp:(progn (org-link-minor-mode) (setq org-link-descriptive t) (org-toggle-link-display))][show links]] [[elisp:(alh-1dex-open)][open 1dex]] [[elisp:(alh-1dex-update)][update 1dex]]
// 
// [[elisp:(alh-set-keywords)][set keywords]] ([[file:~/alh/bin/headeralh][headeralh]])
// emacs-keywords brief="Prefix to the FF emscripten bitcode" freefem javascript start=04/03/15 univ update=20/05/24 written

// FF is not able to give the same results twice so this extra function is required to re-initialize FF between runs (cf
// [[https://groups.google.com/forum/#!msg/emscripten-discuss/NBWEtAHv760/AMGfxaz5oMgJ]]).  This function is called by
// [[file:ffapi.js::ffjs_evaluate]].

function ffjs_callcxx(edp){

  // <<ffjs_ready>>= The initialization of FF is asynchronous so this function is called when it ends
  
  function ffjs_ready(){
    console.log("FreeFem++ loaded and ready to run");

    // [[file:util.cpp::exported_freefem][_exported_freefem_]] call actual C++ function
    freefem(edp_name);

    // <<ffjs_files_data>> [[file:../1dex.org::ffjs_files_data][(i)]] copy resulting files (ALHTODO alhideas1 this makes two copies in memory
    // - on top of the actual graph if there is one. with a web worker we could reduce this to none).
    // cf [[http://kripken.github.io/emscripten-site/docs/api_reference/Filesystem-API.html]]

    for(var f=0;f<ffjs_graphs.length;f++)
      if(ffjs_graphs[f]!="")
	ffjs_files_data[f]=FS.readFile(ffjs_graphs[f]);
  }

  // <<ffjs_output>> [[file:../1dex.org::ffjs_output][(i)]] Redirecting standard streams to the FreeFem++ literate program
  // [[http://kripken.github.io/emscripten-site/docs/api_reference/module.html]]

  var Module = {
    'print':function(text){ffjs_add_stdout(text)},
    'printErr':function(text){ffjs_add_stderr(text)},

    // To avoid "Assertion failed: you need to wait for the runtime to be ready (e.g. wait for main() to be called)". ffjs_ready() needs to
    // be defined in the web page that uses this.
  
    'onRuntimeInitialized':ffjs_ready // [[ffjs_ready][_ffjs_ready_]]
  };

  // --- 8< --- the rest of this function is in [[file:post.js::function_end]]

// Local Variables:
// mode:js
// eval:(visual-line-mode t)
// coding:utf-8
// eval:(flyspell-prog-mode)
// eval:(outline-minor-mode)
// eval:(org-link-minor-mode)
// End:
// LocalWords: emacs
